package router

import (
	"applet/app/admin/hdl"
	categoryHdl "applet/app/admin/hdl/category"
	goodsHdl "applet/app/admin/hdl/goods"
	"applet/app/admin/mw"
	"applet/app/cfg"
	"github.com/gin-gonic/gin"
)

// Init 初始化路由
func Init() *gin.Engine {
	mode := "release"
	if cfg.Debug {
		mode = "debug"
	}
	gin.SetMode(mode)
	//创建一个新的启动器
	r := gin.New()

	// 是否打印访问日志, 在非正式环境都打印
	if mode != "release" {
		r.Use(gin.Logger())
	}
	r.Use(gin.Recovery())

	r.GET("/favicon.ico", func(c *gin.Context) {
		c.Status(204)
	})
	r.NoRoute(func(c *gin.Context) {
		c.JSON(404, gin.H{"code": 404, "msg": "page not found", "data": []struct{}{}})
	})
	r.NoMethod(func(c *gin.Context) {
		c.JSON(405, gin.H{"code": 405, "msg": "method not allowed", "data": []struct{}{}})
	})
	r.Use(mw.Cors)
	AdminRoute(r.Group("/api/admin"))
	IpadInit(r.Group("/api/ipad"))
	return r
}

func rCompany(r *gin.RouterGroup) {
	r.GET("/list", hdl.CompanyList)
	r.POST("/add", hdl.CompanyAdd)
	r.POST("/update", hdl.CompanyUpdate)
	r.DELETE("/delete/:id", hdl.CompanyDelete)
}

func rHomePage(r *gin.RouterGroup) {

}

func rBanner(r *gin.RouterGroup) {

}

func rComm(r *gin.RouterGroup) {
	r.POST("/getMenuList", hdl.MenuList) // 获取菜单栏列表
}

func rOss(r *gin.RouterGroup) {
	r.POST("/upload/token", hdl.ImgReqUpload) // 文件上传获取七牛云上传token
}

func rSetCenter(r *gin.RouterGroup) {
	r.GET("/get", hdl.GetCenter)  // 设置中心-获取
	r.POST("/set", hdl.SetCenter) // 设置中心-设置
}

func rUser(r *gin.RouterGroup) {

}

func rEnterprise(r *gin.RouterGroup) {
	r.GET("/list", hdl.EnterpriseList)
	r.POST("/add", hdl.EnterpriseAdd)
	r.POST("/update", hdl.EnterpriseUpdate)
	r.POST("/updateState", hdl.EnterpriseUpdateState)
	r.POST("/delete", hdl.EnterpriseDelete)
}

func rCategory(r *gin.RouterGroup) {
	r.GET("/list", categoryHdl.List)
	r.GET("/add", categoryHdl.Add)
	r.GET("/del", categoryHdl.Del)
	r.POST("/sort", categoryHdl.Sort)
	r.POST("/save", categoryHdl.Save)
}

func rGoods(r *gin.RouterGroup) {
	r.POST("/del", goodsHdl.DelMallGoods)
	r.POST("/add", goodsHdl.AddMallGoods)
	r.GET("/detail", goodsHdl.MallGoodsDetail)
	r.GET("/list", goodsHdl.GetMallGoodsList)
	r.GET("/checkSkuCode", goodsHdl.CheckSkuCode)
	r.POST("/change-sale-state", goodsHdl.ChangeSaleState)
}

func rRole(r *gin.RouterGroup) {
	r.GET("/roleList", hdl.RoleList)                                //角色列表
	r.POST("/addRole", hdl.AddRole)                                 //角色添加
	r.POST("/roleBindPermissionGroup", hdl.RoleBindPermissionGroup) //角色绑定权限组
	r.POST("/updateRoleState", hdl.UpdateRoleState)                 //修改角色状态
	r.POST("/updateRole", hdl.UpdateRole)                           //修改角色
	r.DELETE("/deleteRole/:id", hdl.DeleteRole)                     //删除角色
	r.GET("/permissionGroupList", hdl.PermissionGroupList)          //权限组列表
	r.POST("/adminList", hdl.AdminList)                             //管理员列表
	r.POST("/updateAdminState", hdl.UpdateAdminState)               //修改管理员状态
	r.POST("/updateAdmin", hdl.UpdateAdmin)                         //修改管理员信息
	r.POST("/addAdmin", hdl.AddAdmin)                               //新增管理员
	r.DELETE("/deleteAdmin/:adm_id", hdl.DeleteAdmin)               //删除管理员
	r.GET("/adminInfo", hdl.AdminInfo)                              //获取管理员信息
	r.POST("/bindAdminRole", hdl.BindAdminRole)                     //绑定角色
}

func AdminRoute(r *gin.RouterGroup) {
	r.GET("/demo", hdl.Demo)

	r.POST("/login", hdl.Login)

	r.Use(mw.Auth)                   //检测登录状态
	r.GET("/userInfo", hdl.UserInfo) //用户信息
	r.GET("/sysCfg", hdl.GetSysCfg)  //基础配置-获取

	rOss(r.Group("/oss"))
	rComm(r.Group("/comm"))

	r.Use(mw.CheckPermission) //检测权限

	rHomePage(r.Group("/homePage"))     //首页
	rCompany(r.Group("/company"))       //公司管理
	rBanner(r.Group("/banner"))         //轮播图管理
	rEnterprise(r.Group("/enterprise")) //校企
	rCategory(r.Group("/category"))     //商品类目
	rGoods(r.Group("/goods"))           //商品

	rRole(r.Group("/role"))           //权限管理
	rSetCenter(r.Group("/setCenter")) //设置中心
	rUser(r.Group("/user"))           //用户管理
}
